function [zero,res,niter]=bisection(fun,a,b,tol,...
                                    nmax,varargin)
%BISECTION Cherche les zros d'une fonction.
%   ZERO=BISECTION(FUN,A,B,TOL,NMAX) tente de trouver
%   un zro ZERO d'une fonction continue FUN sur
%   l'intervalle [A,B] utilisant la mthode de
%   dichotomie (ou bisection).
%   FUN prend des rels  en entre et retourne un
%   scalaire rel. Si la recherche choue, un message
%   d'erreur est affich. FUN peut aussi tre
%   un objet inline,  une fonction anonyme ou
%   bien tre dfinie par un M-files.
%   ZERO=BISECTION(FUN,A,B,TOL,NMAX,P1,P2,...) passe
%   les  paramtres P1, P2,...  la fonction
%   FUN(X,P1,P2,...).
%   [ZERO,RES,NITER]= BISECTION(FUN,...) retourne la
%   valeur approche du zro, la valeur du rsidu en
%   ZERO et le numro de l'itration  laquelle ZERO a
%   t calcul.
x = [a, (a+b)*0.5, b]; fx = feval(fun,x,varargin{:});
if fx(1)*fx(3)>0
 error(['Les signes de la fonction aux extrmits',...
       ' de l''intervalle doivent tre diffrents\n']);
  elseif fx(1) == 0
    zero = a; res = 0; niter = 0; return
elseif fx(3) == 0
    zero = b; res = 0; niter = 0; return
end
niter = 0;
I = (b - a)*0.5;
while I >= tol & niter < nmax
   niter = niter + 1;
   if fx(1)*fx(2) <  0
      x(3) = x(2);
      x(2) = x(1)+(x(3)-x(1))*0.5;
      fx = feval(fun,x,varargin{:});
      I = (x(3)-x(1))*0.5;
   elseif fx(2)*fx(3) < 0
      x(1) = x(2);
      x(2) = x(1)+(x(3)-x(1))*0.5;
      fx = feval(fun,x,varargin{:});
      I = (x(3)-x(1))*0.5;
   else
       x(2) = x(find(fx==0)); I = 0;
   end
end
if  (niter==nmax & I > tol)
   fprintf(['La dichotomie s''est arrte sans \n',...
   'converger avec la tolrance souhaite car \n',...
   'le nombre maximal d''itrations a t atteint\n']);
end
zero = x(2); x = x(2); res = feval(fun,x,varargin{:});
return
