function [xh,uh]=newmarkwave(xspan,tspan,nstep,param,...
               c,u0,v0,g,f,varargin)
%NEWMARKWAVE rsout l'quation des ondes avec la
% mthode de Newmark.
% [XH,UH]=NEWMARKWAVE(XSPAN,TSPAN,NSTEP,PARAM,C,...
% U0,V0,G,F) rsout l'quation des ondes
% D^2 U/DT^2 - C D^2U/DX^2 = F
% dans ]XSPAN(1),XSPAN(2)[ x ]TSPAN(1),TSPAN(2)[ en
% utilisant la mthode de Newmark avec les conditions
% initiales U(X,0)=U0(X), DU/DX(X,0)=V0(X) et les
% conditions de Dirichlet U(X,T)=G(X,T) pour X=XSPAN(1)
% et X=XSPAN(2). C est une constante positive.
% NSTEP(1) est le nombre de pas d'intgration en
% espace, NSTEP(2) est le nombre de pas d'intgration
% en temps. PARAM(1)=ZETA et  PARAM(2)=THETA.
% U0(X), V0(X), G(X,T) et F(x,T) sont des fonctions
% inline, anonymes ou dfinies par un M-file.
% XH contient les noeuds de discrtisation.
% UH contient la solution numrique au temps TSPAN(2).}
% [XH,UH]=NEWMARKWAVE(XSPAN,TSPAN,NSTEP,PARAM,C,...
% U0,V0,G,F,P1,P2,...) passe les paramtres
% supplmentaires  P1, P2,... aux fonctions U0,V0,G,F.
h  = (xspan(2)-xspan(1))/nstep(1);
dt = (tspan(2)-tspan(1))/nstep(2);
zeta = param(1);  theta = param(2);
N = nstep(1)+1;
e = ones(N,1); D = spdiags([e -2*e e],[-1,0,1],N,N);
I = speye(N); lambda = dt/h;
A = I-c*lambda^2*zeta*D;
An = I+c*lambda^2*(0.5-zeta)*D;
A(1,:) = 0; A(1,1) = 1; A(N,:) = 0; A(N,N) = 1;
xh = (linspace(xspan(1),xspan(2),N))';
fn = feval(f,xh,tspan(1),varargin{:});
un = feval(u0,xh,varargin{:});
vn = feval(v0,xh,varargin{:});
[L,U]=lu(A);
alpha = dt^2*zeta; beta = dt^2*(0.5-zeta);
theta1 = 1-theta;
for t = tspan(1)+dt:dt:tspan(2)
    fn1 = feval(f,xh,t,varargin{:});
    rhs = An*un+dt*I*vn+alpha*fn1+beta*fn;
    temp = feval(g,[xspan(1),xspan(2)],t,varargin{:});
    rhs([1,N]) = temp;
    uh = L\rhs;    uh = U\uh;
    v = vn + dt*((1-theta)*(c*D*un/h^2+fn)+...
        theta*(c*D*uh/h^2+fn1));
    fn = fn1;    un = uh;    vn = v;
end
